@@ -40,6 +40,7 @@
 #include <libgnomevfs/gnome-vfs.h>
 
 #include <libgnome/gnome-desktop-item.h>
+#include <gmenu-tree.h>
 
 #include <bonobo/bonobo-exception.h>
 #include <bonobo-activation/bonobo-activation.h>
@@ -286,9 +287,10 @@ async_update (GnomeVFSAsyncHandle *handle, GnomeVFSXferProgressInfo *info, gpoin
 	if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
 		d(g_print ("Transfer finished\n"));
 		end_bugzilla_download (count);
+		return FALSE;
 	}
 
-	return druid_data.download_in_progress;
+	return TRUE;
 }
 
 /**
@@ -635,9 +637,6 @@ load_bugzilla_xml (void)
 	d(g_print ("loading xml..\n"));
 
 	g_hash_table_foreach (druid_data.bugzillas, (GHFunc)load_bugzilla_xml_cb, NULL);
-
-	if (druid_data.show_products)
-		products_list_load ();
 }
 
 static void
@@ -860,7 +859,6 @@ create_components_list (void)
 void
 end_bugzilla_download (int count)
 {
-	druid_data.download_in_progress = FALSE;
 	if (count>0)
 		gtk_dialog_response (GTK_DIALOG (GET_WIDGET ("progress-dialog")),
 				     DOWNLOAD_FINISHED_OK_RESPONSE);
@@ -873,7 +871,6 @@ end_bugzilla_download (int count)
 gboolean
 start_bugzilla_download (void)
 {
-	druid_data.download_in_progress = TRUE;
 	gnome_vfs_async_xfer (	    
 		&druid_data.vfshandle,
 		druid_data.dlsources,
@@ -892,26 +889,18 @@ void
 load_bugzillas (void)
 {
 	BugzillaBTS *bts;
-	GtkWidget *w;
 	DIR *dir;
 	struct dirent *dent;
 	char *p;
 
 	dir = opendir (BUDDY_DATADIR"/bugzilla/");
 	if (!dir) {
-		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
-					    0,
-					    GTK_MESSAGE_ERROR,
-					    GTK_BUTTONS_OK,
-					    _("Bug Buddy could not open '%s'.\n"
-					      "Please make sure Bug Buddy was "
-					      "installed correctly.\n\n"
-					      "Bug Buddy will now quit."),
-					    BUDDY_DATADIR "/bugzilla/");
-		gtk_dialog_set_default_response (GTK_DIALOG (w),
-						 GTK_RESPONSE_OK);
-		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_destroy (w);
+		buddy_error (GTK_WIDGET ("druid-window"), 
+			     _("Bug Buddy could not open '%s'.\n"
+			       "Please make sure Bug Buddy was "
+			       "installed correctly.\n\n"
+			       "Bug Buddy will now quit."),
+			     BUDDY_DATADIR "/bugzilla/");
 		exit (0);
 	}
 
@@ -944,19 +933,12 @@ load_bugzillas (void)
 	closedir (dir);
 
 	if (druid_data.need_to_download && !druid_data.dlsources) {
-		w = gtk_message_dialog_new (GTK_WINDOW (GET_WIDGET ("druid-window")),
-					    0,
-					    GTK_MESSAGE_ERROR,
-					    GTK_BUTTONS_OK,
-					    _("Bug Buddy could not find any information on "
-					      "where to submit bugs.\n\n"
-					      "Please make sure Bug Buddy was "
-					      "installed correctly.\n\n"
-					      "Bug Buddy will now quit."));
-		gtk_dialog_set_default_response (GTK_DIALOG (w),
-						 GTK_RESPONSE_OK);
-		gtk_dialog_run (GTK_DIALOG (w));
-		gtk_widget_destroy (w);
+		buddy_error (GET_WIDGET ("druid-window"),
+			     _("Bug Buddy could not find any information on "
+			       "where to submit bugs.\n\n"
+			       "Please make sure Bug Buddy was "
+			       "installed correctly.\n\n"
+			       "Bug Buddy will now quit."));
 		exit (0);
 	}
 
@@ -966,11 +948,10 @@ load_bugzillas (void)
 	d(g_list_foreach (druid_data.dldests, (GFunc)p_string, NULL));
 }
 
-#define ALL_APPLICATIONS_URI "applications-all-users:///"
 #define APPLET_REQUIREMENTS \
 	"has_all (repo_ids, ['IDL:Bonobo/Control:1.0'," \
 	"		     'IDL:GNOME/Vertigo/PanelAppletShell:1.0']) && " \
-	"defined (panel:icon) && defined (panel:category)"
+	"defined (panel:icon)"
 
 #define BUGZILLA_BUGZILLA               "X-GNOME-Bugzilla-Bugzilla"
 #define BUGZILLA_PRODUCT                "X-GNOME-Bugzilla-Product"
@@ -999,9 +980,6 @@ bugzilla_application_new (const char *name,
 			  const char *other_programs)
 {
 	BugzillaApplication *app;
-	GdkPixbuf *pb = NULL;
-	char *icon_copy;
-	GError *error = NULL;
 	char **programv;
 	int i;
 
@@ -1015,27 +993,34 @@ bugzilla_application_new (const char *name,
 	app->version     = g_strdup (version);
 	app->email       = g_strdup (email);
 
-	if (icon && *icon != G_DIR_SEPARATOR)
-		icon_copy = gnome_program_locate_file (NULL,
-						       GNOME_FILE_DOMAIN_PIXMAP,
-						       icon, TRUE, NULL);
-	else
-		icon_copy = g_strdup (icon);
-
-	if (icon_copy) {
-		pb = gdk_pixbuf_new_from_file (icon_copy, &error);
-		g_free (icon_copy);
+	if (icon) {
+		if (g_path_is_absolute (icon)) {
+			app->pixbuf = gdk_pixbuf_new_from_file_at_size (icon, 24, 24, NULL);
+		}
+		if (app->pixbuf == NULL) {
+			app->pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								icon, 24, 0, NULL);
+		}
+		if (app->pixbuf == NULL && strrchr (icon, '.') != NULL) {
+			char *name;
+			name = g_strndup (icon, strlen (icon) - strlen (strrchr (icon, '.')));
+			app->pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								name, 24, 0, NULL);
+			g_free (name);
+		}
+	
+		if (app->pixbuf == NULL) {
+			char *filename;
+			filename = g_strdup_printf (BUDDY_ICONDIR"/pixmaps/%s", icon);
+			app->pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 24, 24, NULL);
+			g_free (filename);
+		}
 	}
 
-	if (pb) {
-		app->pixbuf = gdk_pixbuf_scale_simple (pb, 20, 20, GDK_INTERP_HYPER);
-		g_object_unref (pb);
-	} else if (error) {
-		g_warning (_("Couldn't load icon for %s: %s"), app->name, error->message);
-		g_error_free (error);
+	if (app->pixbuf == NULL) {
+		g_warning (_("Couldn't load icon for %s"), app->name);
 	}
 	
-
 	if (program) {
 		g_shell_parse_argv (program, &i, &programv, NULL);
 		if (programv[0]) {
@@ -1120,6 +1105,7 @@ NULL)) {
 			continue;
 		}
 
+
 		bugzilla_application_new (
 			bonobo_server_info_prop_lookup (info, "name", langs),
 			bonobo_server_info_prop_lookup (info, "description", langs),
@@ -1136,83 +1122,162 @@ NULL)) {
 	CORBA_free (info_list);
 }
 
-static gboolean
-visit_cb (const char *rel_path,
-	  GnomeVFSFileInfo *info,
-	  gboolean recursing_will_loop,
-	  gpointer data,
-	  gboolean *recurse)
+static int
+compare_applications (GMenuTreeEntry *a,
+		      GMenuTreeEntry *b)
 {
-	GnomeDesktopItem *ditem;
-	GError *error = NULL;
-	char *full_path, *icon;
-	GnomeIconTheme *git = data;
-	
-	if (info->name[0] != '.'
-	    || (info->name[1] != '.' && info->name[1] != 0)
-	    || info->name[2] != 0) {
-		if (recursing_will_loop) {
-			g_print ("Loop detected\n");
-			return TRUE;
-		}
-		*recurse = TRUE;
-	} else {
-		*recurse = FALSE;
-	}
-
-	full_path = g_strconcat (ALL_APPLICATIONS_URI, rel_path, NULL);
-
-	ditem = gnome_desktop_item_new_from_uri (full_path, 0, &error);
-	if (!ditem) {
-		if (error) {
-			g_warning ("Couldn't load %s: %s", full_path, error->message);
-		}
-		goto visit_cb_out;
-	}
-    
-	if ((gnome_desktop_item_get_entry_type (ditem) != GNOME_DESKTOP_ITEM_TYPE_APPLICATION) || !gnome_desktop_item_get_string (ditem, BUGZILLA_BUGZILLA))
-		goto visit_cb_out;
-
-	icon = gnome_desktop_item_get_icon (ditem, git);
-	bugzilla_application_new (
-		gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME),
-		gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_BUGZILLA),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_PRODUCT),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_COMPONENT),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_VERSION),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_EMAIL),
-		icon,
-		gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC),
-		gnome_desktop_item_get_string (ditem, BUGZILLA_OTHER_BINARIES));
-	g_free (icon);
-
- visit_cb_out:
-	g_free (full_path);
-	if (error)
-		g_error_free (error);
-	if (ditem)
-		gnome_desktop_item_unref (ditem);
-
-	return TRUE;
+	return g_utf8_collate (gmenu_tree_entry_get_name (a),
+			       gmenu_tree_entry_get_name (b));
 }
 
+static GSList *get_all_applications_from_dir (GMenuTreeDirectory *directory,
+					      GSList             *list);
+
+static GSList *
+get_all_applications_from_alias (GMenuTreeAlias *alias,
+				 GSList         *list)
+{
+	GMenuTreeItem *aliased_item;
+
+	aliased_item = gmenu_tree_alias_get_item (alias);
+
+	switch (gmenu_tree_item_get_type (aliased_item)) {
+	case GMENU_TREE_ITEM_DIRECTORY:
+		list = get_all_applications_from_dir (GMENU_TREE_DIRECTORY (aliased_item), list);
+		break;
+
+	case GMENU_TREE_ITEM_ENTRY:
+		list = g_slist_append (list, gmenu_tree_item_ref (aliased_item));
+		break;
+
+	default:
+		break;
+	}
+
+	gmenu_tree_item_unref (aliased_item);
+
+	return list;
+}
+
+static GSList *
+get_all_applications_from_dir (GMenuTreeDirectory *directory,
+			       GSList             *list)
+{
+	GSList *items;
+	GSList *l;
+
+	items = gmenu_tree_directory_get_contents (directory);
+	for (l = items; l; l = l->next) {
+		GMenuTreeItem *item = l->data;
+
+		switch (gmenu_tree_item_get_type (item)) {
+		case GMENU_TREE_ITEM_DIRECTORY:
+			list = get_all_applications_from_dir (GMENU_TREE_DIRECTORY (item), list);
+			break;
+
+		case GMENU_TREE_ITEM_ENTRY:
+			list = g_slist_append (list, gmenu_tree_item_ref (item));
+			break;
+
+		case GMENU_TREE_ITEM_ALIAS:
+			list = get_all_applications_from_alias (GMENU_TREE_ALIAS (item), list);
+			break;
+
+		default:
+			break;
+		}
+
+		gmenu_tree_item_unref (item);
+	}
+
+	g_slist_free (items);
+
+	return list;
+}
+
+static GSList *
+get_all_applications (void)
+{
+	GMenuTree          *tree;
+	GMenuTreeDirectory *root;
+	GSList             *retval;
+
+	tree = gmenu_tree_lookup ("applications.menu", GMENU_TREE_FLAGS_NONE);
+
+	root = gmenu_tree_get_root_directory (tree);
+
+	retval = get_all_applications_from_dir (root, NULL);
+
+	gmenu_tree_item_unref (root);
+	gmenu_tree_unref (tree);
+
+	retval = g_slist_sort (retval, (GCompareFunc) compare_applications);
+
+	return retval;
+}
+
+
 void
 load_applications (void)
 {
 	GnomeIconTheme *git;
+	GSList         *all_applications;
+	GSList         *l;
+	const char     *prev_name = NULL;
+	GError	       *error = NULL;
 
 	druid_data.program_to_application = g_hash_table_new (g_str_hash, g_str_equal);
 
 	git = gnome_icon_theme_new ();
 	gnome_icon_theme_set_allow_svg (git, FALSE);
-	
-	/* load applications */
-	gnome_vfs_directory_visit (ALL_APPLICATIONS_URI,
-				   GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
-				   GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK,
-				   visit_cb, git);
 
+	all_applications = get_all_applications ();
+	for (l = all_applications; l; l = l->next) {
+		GnomeDesktopItem *ditem;
+		char *icon;
+		GMenuTreeEntry *entry = l->data;
+
+		if (prev_name && strcmp (gmenu_tree_entry_get_name (entry), prev_name) == 0) {
+			gmenu_tree_item_unref (entry);
+			continue;
+		}
+		ditem = gnome_desktop_item_new_from_uri (gmenu_tree_entry_get_desktop_file_path (entry), 
+							 0, &error);
+		if (!ditem) {
+			if (error) {
+				g_warning ("Couldn't load %s: %s", gmenu_tree_entry_get_desktop_file_path (entry),
+					   error->message);
+				g_error_free (error);
+				error = NULL;
+			}
+			gmenu_tree_item_unref (entry);
+			continue;
+		}
+    
+		if ((gnome_desktop_item_get_entry_type (ditem) != GNOME_DESKTOP_ITEM_TYPE_APPLICATION) || !gnome_desktop_item_get_string (ditem, BUGZILLA_BUGZILLA)) {
+			gnome_desktop_item_unref (ditem);	
+			gmenu_tree_item_unref (entry);
+			continue;
+		}
+
+		icon = gnome_desktop_item_get_icon (ditem, git);
+		bugzilla_application_new (
+			gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME),
+			gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_BUGZILLA),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_PRODUCT),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_COMPONENT),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_VERSION),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_EMAIL),
+			icon,
+			gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC),
+			gnome_desktop_item_get_string (ditem, BUGZILLA_OTHER_BINARIES));
+		g_free (icon);
+		prev_name = gmenu_tree_entry_get_name (entry);
+		gnome_desktop_item_unref (ditem);	
+		gmenu_tree_item_unref (entry);
+	}
+	g_slist_free (all_applications);
 
 	load_applets (git);
 
@@ -1347,8 +1412,6 @@ products_list_load (void)
 	druid_data.product = NULL;
 
 	if (druid_data.show_products) {
-		if (druid_data.download_in_progress)
-			return;
 		g_hash_table_foreach (druid_data.bugzillas, (GHFunc)add_products, store);
 	} else {
 		g_slist_foreach (druid_data.applications, (GFunc)add_application, store);
